/*
 * SPDX-FileCopyrightText: Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* All structs in this file should be packed to make sure no aligment issues will occure.
 * Please align all structs to 64 bits.
 */

#ifndef __FLEXIO_COMMON_H__
#define __FLEXIO_COMMON_H__

#include <stdint.h>

typedef uint64_t flexio_uintptr_t;

struct flexio_hw_eq {
	uint32_t eq_num;
	uint32_t reserved;                /* Alignment for 8B */
	flexio_uintptr_t eq_ring_daddr;
} __attribute__((__packed__, aligned(8)));

struct flexio_hw_cq {
	uint32_t cq_num;
	uint32_t reserved;                /* Alignment for 8B */
	flexio_uintptr_t cq_ring_daddr;
	flexio_uintptr_t cq_dbr_daddr;
} __attribute__((__packed__, aligned(8)));

struct flexio_hw_wq {
	uint32_t wq_num;
	uint32_t data_buff_mkey;
	flexio_uintptr_t wq_ring_daddr;
	flexio_uintptr_t wq_dbr_daddr;
	flexio_uintptr_t wq_tx_buff;
} __attribute__((__packed__, aligned(8)));

struct flexio_hw_qp {
	uint32_t qp_num;
	uint32_t sqd_buff_mkey;
	uint32_t rqd_buff_mkey;
	uint32_t qp_sq_offset;
	uint32_t qp_sq_wqe_shift;
	uint32_t qp_sq_log_wqe_cnt;
	uint32_t qp_rq_wqe_shift;
	uint32_t qp_rq_log_wqe_cnt;

	flexio_uintptr_t qp_wq_buffer_daddr;
	flexio_uintptr_t qp_dbr_daddr;
	flexio_uintptr_t sqd_daddr;
	flexio_uintptr_t rqd_daddr;
} __attribute__((__packed__, aligned(8)));

struct flexio_rpc_poll {
	uint64_t flag;
	uint64_t return_value;
} __attribute__((__packed__, aligned(8)));

struct flexio_rpc_data {
	uint32_t poll_lkey;
	uint32_t window_id;
	uint64_t poll_haddr;
	uint64_t entry_point;
	uint64_t arg1;
	uint64_t arg2;
	uint64_t arg3;
} __attribute__((__packed__, aligned(8)));

struct flexio_printcfg_data {
	uint32_t qp_num;
	uint32_t cq_num;

	uint32_t dev_lkey;
	uint32_t outbox_id;

	uint8_t log_dev_rings_depth;
	uint8_t log_data_chunk_bsize;

	/* HOST data relevant for RDMA write (async mode) */
	uint8_t is_async;
	uint8_t log_host_data_bsize;
	uint32_t host_lkey;
	uint32_t qp_sq_offset;
	uint32_t reserved;
	uint64_t host_data_haddr;

	flexio_uintptr_t qp_wq_buffer_daddr;
	flexio_uintptr_t sqd_daddr;
	flexio_uintptr_t qp_dbr_daddr;
	flexio_uintptr_t cq_dbr_daddr;
	flexio_uintptr_t service_pi_ring_daddr;
} __attribute__((__packed__, aligned(8)));

struct flexio_thread_metadata {
	uint32_t thread_id;
	uint32_t reserved;

	flexio_uintptr_t thread_local_storage_daddr;
} __attribute__((__packed__, aligned(8)));

#endif
